'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames2 = require('classnames');

var _classnames3 = _interopRequireDefault(_classnames2);

var _constants = require('./constants');

var _context = require('./context');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FormItem = function (_Component) {
  _inherits(FormItem, _Component);

  function FormItem() {
    _classCallCheck(this, FormItem);

    return _possibleConstructorReturn(this, (FormItem.__proto__ || Object.getPrototypeOf(FormItem)).apply(this, arguments));
  }

  _createClass(FormItem, [{
    key: 'getHelpMsg',
    value: function getHelpMsg() {
      var help = this.props.help;
      var form = this.context.form;


      if (help === undefined && this.getOnlyControl()) {
        var _getField = this.getField(),
            errors = _getField.errors;

        if (errors) {
          return errors.map(function (e, index) {
            var node = null;
            if (_react2.default.isValidElement(e)) {
              node = e;
            } else if (_react2.default.isValidElement(e.message)) {
              node = e.message;
            }

            return node ? _react2.default.cloneElement(node, { key: index }) : e.message;
          });
        }
        return '';
      }

      return help;
    }
  }, {
    key: 'getControls',
    value: function getControls() {
      var children = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props.children;
      var recursively = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      var controls = [];
      var childrenArray = _react2.default.Children.toArray(children);

      for (var i = 0; i < childrenArray.length; i++) {
        if (!recursively && controls.length > 0) {
          break;
        }

        var child = childrenArray[i];

        if (child.type && (child.type === FormItem || child.type.displayName === 'FormItem')) {
          continue;
        }

        if (!child.props) {
          continue;
        }

        if (_constants.FIELD_META_PROP in child.props) {
          controls.push(child);
        } else if (child.props.children) {
          controls = controls.concat(this.getControls(child.props.children, recursively));
        }
      }

      return controls;
    }
  }, {
    key: 'getOnlyControl',
    value: function getOnlyControl() {
      var child = this.getControls(this.props.children, false)[0];
      return child !== undefined ? child : null;
    }
  }, {
    key: 'getChildProp',
    value: function getChildProp(prop) {
      var child = this.getOnlyControl();

      if (!child) {
        return null;
      }

      return child && child.props && child.props[prop];
    }
  }, {
    key: 'getId',
    value: function getId() {
      return this.getChildProp('id');
    }
  }, {
    key: 'getField',
    value: function getField() {
      return this.getChildProp(_constants.FIELD_DATA_PROP);
    }
  }, {
    key: 'renderHelp',
    value: function renderHelp(context) {
      var prefixCls = context.prefixCls;


      var help = this.getHelpMsg();

      return help ? _react2.default.createElement(
        'div',
        { className: prefixCls + '-explain' },
        help.length ? help[0] : ''
      ) : null;
    }
  }, {
    key: 'renderLabel',
    value: function renderLabel(context) {
      var label = this.props.label;
      var prefixCls = context.prefixCls,
          labelAlign = context.labelAlign;


      return label ? _react2.default.createElement(
        'div',
        { className: prefixCls + '-item-label ' + prefixCls + '-item-label-' + labelAlign },
        label
      ) : null;
    }
  }, {
    key: 'renderChildren',
    value: function renderChildren(context) {
      var children = this.props.children;
      var prefixCls = context.prefixCls;


      if (!children) {
        return null;
      }

      children = _react2.default.Children.map(children, function (child) {
        if (child && typeof child.type === 'function' && !child.props.size) {
          return _react2.default.cloneElement(child, { size: 'large' });
        }
        return child;
      });

      return _react2.default.createElement(
        'div',
        { className: prefixCls + '-item-content' },
        children
      );
    }
  }, {
    key: 'renderDescription',
    value: function renderDescription(context) {
      var description = this.props.description;
      var prefixCls = context.prefixCls;


      if (!description) {
        return null;
      }

      return _react2.default.createElement(
        'div',
        { className: prefixCls + '-item-description' },
        description
      );
    }
  }, {
    key: 'renderItem',
    value: function renderItem(context) {
      var _classnames;

      var _props = this.props,
          className = _props.className,
          hasArrow = _props.hasArrow,
          style = _props.style;
      var prefixCls = context.prefixCls;

      var children = this.renderChildren(context);

      var itemClassName = (0, _classnames3.default)((_classnames = {}, _defineProperty(_classnames, prefixCls + '-item', true), _defineProperty(_classnames, prefixCls + '-item-help', !!this.getHelpMsg()), _defineProperty(_classnames, '' + className, !!className), _classnames));

      var arrowRender = hasArrow && _react2.default.createElement('div', { className: prefixCls + '-arrow' });

      return _react2.default.createElement(
        'div',
        { className: itemClassName, style: style },
        _react2.default.createElement(
          'div',
          { className: prefixCls + '-item-inner' },
          this.renderLabel(context),
          children,
          this.renderDescription(context),
          arrowRender
        ),
        this.renderHelp(context)
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      return _react2.default.createElement(
        _context.FormContext.Consumer,
        null,
        function (context) {
          return _this2.renderItem(context);
        }
      );
    }
  }]);

  return FormItem;
}(_react.Component);

FormItem.defaultProps = {
  hasArrow: false
};
FormItem.propTypes = {
  prefixCls: _propTypes2.default.string,
  label: _propTypes2.default.node,
  description: _propTypes2.default.node,
  className: _propTypes2.default.string,
  id: _propTypes2.default.string,
  children: _propTypes2.default.node,
  hasArrow: _propTypes2.default.bool,
  style: _propTypes2.default.any
};
exports.default = FormItem;